/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.importer;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.dive4elements.river.importer.common.StoreMode;
import org.dive4elements.river.model.HYK;
import org.dive4elements.river.model.River;
import org.hibernate.Query;
import org.hibernate.Session;

public class ImportHYK
{
    private static Logger log = Logger.getLogger(ImportHYK.class);

    protected ImportRiver river;
    protected String      description;

    protected List<ImportHYKEntry> entries;

    protected StoreMode storeMode;

    protected HYK peer;

    public ImportHYK() {
        this.entries = new ArrayList<>();
        this.storeMode = StoreMode.NONE;
    }

    public ImportHYK(final ImportRiver river, final String description) {
        this();
        this.river       = river;
        this.description = description;
    }

    public ImportRiver getRiver() {
        return this.river;
    }

    public void setRiver(final ImportRiver river) {
        this.river = river;
    }

    public void addEntry(final ImportHYKEntry entry) {
        this.entries.add(entry);
        entry.setHYK(this);
    }

    public void storeDependencies() {
        log.info("store HYK '" + this.description + "'");
        getPeer();
        for (int i = 0, N = this.entries.size(); i < N; ++i) {
            final ImportHYKEntry entry = this.entries.get(i);
            log.info("  store km " + entry.getKm() +
                    " (" + (i+1) + " of " + N + ")");
            entry.storeDependencies();
        }
    }

    public HYK getPeer() {
        if (this.peer == null) {
            final River r = this.river.getPeer();
            final Session session = ImporterSession.getInstance()
                    .getDatabaseSession();
            final Query query = session.createQuery(
                    "from HYK where river=:river and description=:description");
            query.setParameter("river", r);
            query.setParameter("description", this.description);
            final List<HYK> hyks = query.list();
            if (hyks.isEmpty()) {
                this.peer = new HYK(r, this.description);
                session.save(this.peer);
                this.storeMode = StoreMode.INSERT;
            }
            else {
                this.peer = hyks.get(0);
                this.storeMode = StoreMode.UPDATE;
            }
        }
        return this.peer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
